"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declarationTitle = void 0;
const typedoc_1 = require("typedoc");
const escape_1 = require("./escape");
const member_symbol_1 = require("./member-symbol");
const strip_comments_1 = require("./strip-comments");
const strip_line_breaks_1 = require("./strip-line-breaks");
const type_1 = require("./type");
function declarationTitle() {
    const md = [member_symbol_1.memberSymbol.call(this) + ' '];
    if (this.flags && !this.flags.isRest) {
        md.push(this.flags.map((flag) => `\`${flag}\``).join(' '));
    }
    md.push(`${this.flags.isRest ? '... ' : ' '}**${escape_1.escape(this.name)}**`);
    if (this instanceof typedoc_1.DeclarationReflection && this.typeParameters) {
        md.push(`\\<${this.typeParameters
            .map((typeParameter) => typeParameter.name)
            .join(', ')}>`);
    }
    md.push(`: ${getType(this)}`);
    if (this.defaultValue) {
        md.push(` = ${strip_line_breaks_1.stripLineBreaks(escape_1.escape(strip_comments_1.stripComments(this.defaultValue)))}`);
    }
    return md.join('');
}
exports.declarationTitle = declarationTitle;
function getType(reflection) {
    return type_1.type.call(reflection.type ? reflection.type : reflection, shouldCollapse(reflection));
}
function shouldCollapse(reflection) {
    if (reflection.kind === typedoc_1.ReflectionKind.ObjectLiteral) {
        return true;
    }
    if (reflection.kind === typedoc_1.ReflectionKind.Variable) {
        const type = reflection.type;
        if (type.declaration && type.declaration.signatures) {
            return false;
        }
        return true;
    }
    return false;
}
