"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const Handlebars = require("handlebars");
const typedoc_1 = require("typedoc");
const events_1 = require("typedoc/dist/lib/output/events");
const theme_1 = require("typedoc/dist/lib/output/theme");
const DefaultTheme_1 = require("typedoc/dist/lib/output/themes/DefaultTheme");
const breadcrumbs_component_1 = require("./components/breadcrumbs.component");
const comments_component_1 = require("./components/comments.component");
const utils_component_1 = require("./components/utils.component");
class MarkdownTheme extends theme_1.Theme {
    constructor(renderer, basePath) {
        super(renderer, basePath);
        this.listenTo(renderer, events_1.PageEvent.END, this.onPageEnd, 1024);
        renderer.removeComponent('assets');
        renderer.removeComponent('javascript-index');
        renderer.removeComponent('toc');
        renderer.removeComponent('pretty-print');
        renderer.removeComponent('marked-links');
        renderer.removeComponent('legend');
        renderer.removeComponent('navigation');
        renderer.addComponent('comments', new comments_component_1.CommentsComponent(renderer));
        renderer.addComponent('breadcrumbs', new breadcrumbs_component_1.BreadcrumbsComponent(renderer));
        renderer.addComponent('utils', new utils_component_1.HelperUtilsComponent(renderer));
    }
    static formatContents(contents) {
        return (contents
            .replace(/[\r\n]{3,}/g, '\n\n')
            .replace(/!spaces/g, '')
            .replace(/^\s+|\s+$/g, '') + '\n');
    }
    isOutputDirectory(outputDirectory) {
        let isOutputDirectory = true;
        const listings = fs.readdirSync(outputDirectory);
        listings.forEach((listing) => {
            if (!this.allowedDirectoryListings(this.entryFile).includes(listing)) {
                isOutputDirectory = false;
                return;
            }
        });
        return isOutputDirectory;
    }
    allowedDirectoryListings(entryFileName) {
        return [
            entryFileName,
            'README.md',
            'globals.md',
            'classes',
            'enums',
            'interfaces',
            'modules',
            'media',
            '.DS_Store',
        ];
    }
    getUrls(project) {
        const urls = [];
        const entryPoint = this.getEntryPoint(project);
        const omitReadme = this.readme === 'none';
        if (omitReadme) {
            entryPoint.url = this.entryFile;
            urls.push(new typedoc_1.UrlMapping(this.entryFile, Object.assign({}, entryPoint), 'reflection.hbs'));
        }
        else {
            entryPoint.url = 'globals.md';
            urls.push(new typedoc_1.UrlMapping('globals.md', entryPoint, 'reflection.hbs'));
            urls.push(new typedoc_1.UrlMapping(this.entryFile, project, 'index.hbs'));
        }
        if (entryPoint.children) {
            entryPoint.children.forEach((child) => {
                if (child instanceof typedoc_1.DeclarationReflection) {
                    this.buildUrls(child, urls);
                }
            });
        }
        return urls;
    }
    buildUrls(reflection, urls) {
        const mapping = DefaultTheme_1.DefaultTheme.getMapping(reflection);
        if (mapping) {
            if (!reflection.url || !DefaultTheme_1.DefaultTheme.URL_PREFIX.test(reflection.url)) {
                const url = this.toUrl(mapping, reflection);
                urls.push(new typedoc_1.UrlMapping(url, reflection, mapping.template));
                reflection.url = url;
                reflection.hasOwnDocument = true;
            }
            for (const child of reflection.children || []) {
                if (mapping.isLeaf) {
                    this.applyAnchorUrl(child, reflection);
                }
                else {
                    this.buildUrls(child, urls);
                }
            }
        }
        else if (reflection.parent) {
            this.applyAnchorUrl(reflection, reflection.parent);
        }
        return urls;
    }
    toUrl(mapping, reflection) {
        return mapping.directory + '/' + this.getUrl(reflection) + '.md';
    }
    getUrl(reflection, relative, separator = '.') {
        let url = reflection.getAlias();
        if (reflection.parent &&
            reflection.parent !== relative &&
            !(reflection.parent instanceof typedoc_1.ProjectReflection)) {
            url =
                this.getUrl(reflection.parent, relative, separator) + separator + url;
        }
        return url;
    }
    applyAnchorUrl(reflection, container) {
        if (!reflection.url || !DefaultTheme_1.DefaultTheme.URL_PREFIX.test(reflection.url)) {
            const reflectionId = reflection.name.toLowerCase();
            const anchor = this.toAnchorRef(reflectionId);
            reflection.url = container.url + '#' + anchor;
            reflection.anchor = anchor;
            reflection.hasOwnDocument = false;
        }
        reflection.traverse((child) => {
            if (child instanceof typedoc_1.DeclarationReflection) {
                this.applyAnchorUrl(child, container);
            }
        });
    }
    toAnchorRef(reflectionId) {
        return reflectionId;
    }
    getEntryPoint(project) {
        const entryPoint = this.owner.entryPoint;
        if (entryPoint) {
            const reflection = project.getChildByName(entryPoint);
            if (reflection) {
                if (reflection instanceof typedoc_1.ContainerReflection) {
                    return reflection;
                }
                else {
                    this.application.logger.warn('The given entry point `%s` is not a container.', entryPoint);
                }
            }
            else {
                this.application.logger.warn('The entry point `%s` could not be found.', entryPoint);
            }
        }
        return project;
    }
    getNavigation(project) {
        var _a, _b, _c;
        const entryPoint = this.getEntryPoint(project);
        const hasSeperateGlobals = this.readme !== 'none';
        const navigation = createNavigationItem(project.name);
        (_a = navigation.children) === null || _a === void 0 ? void 0 : _a.push(createNavigationItem(hasSeperateGlobals ? 'README' : 'Globals', this.entryFile));
        if (hasSeperateGlobals) {
            (_b = navigation.children) === null || _b === void 0 ? void 0 : _b.push(createNavigationItem('Globals', 'globals.md'));
        }
        if (entryPoint.groups) {
            buildGroups(entryPoint.groups);
        }
        (_c = navigation.children) === null || _c === void 0 ? void 0 : _c.sort(sortCallback);
        function buildGroups(groups, level = 0) {
            groups.forEach((reflectionGroup) => {
                var _a, _b;
                if (reflectionGroup.allChildrenHaveOwnDocument()) {
                    let reflectionGroupItem = (_a = navigation.children) === null || _a === void 0 ? void 0 : _a.find((child) => child.title === reflectionGroup.title);
                    if (!reflectionGroupItem) {
                        reflectionGroupItem = createNavigationItem(reflectionGroup.title);
                        (_b = navigation.children) === null || _b === void 0 ? void 0 : _b.push(reflectionGroupItem);
                    }
                    reflectionGroup.children.forEach((reflectionGroupChild) => {
                        var _a;
                        const reflectionGroupChildItem = createNavigationItem(reflectionGroupChild.getFullName().replace(/\"/g, ''), reflectionGroupChild.url);
                        if (reflectionGroupItem) {
                            (_a = reflectionGroupItem.children) === null || _a === void 0 ? void 0 : _a.push(reflectionGroupChildItem);
                        }
                        const reflection = reflectionGroupChild;
                        if (reflection.groups) {
                            buildGroups(reflection.groups, level + 1);
                        }
                    });
                }
            });
        }
        function createNavigationItem(title, url) {
            const navigationItem = new typedoc_1.NavigationItem(title.replace(/\"/g, ''), url);
            navigationItem.children = [];
            delete navigationItem.reflection;
            delete navigationItem.isLabel;
            return navigationItem;
        }
        function sortCallback(a, b) {
            const weights = {
                Namespaces: 1,
                Enumerations: 2,
                Classes: 3,
                Interfaces: 4,
            };
            const aWeight = weights[a.title] || 0;
            const bWeight = weights[b.title] || 0;
            return aWeight - bWeight;
        }
        return navigation;
    }
    onPageEnd(page) {
        page.contents = page.contents
            ? MarkdownTheme.formatContents(page.contents)
            : '';
    }
    get entryFile() {
        return 'README.md';
    }
}
MarkdownTheme.HANDLEBARS = Handlebars.create();
__decorate([
    typedoc_1.BindOption('readme')
], MarkdownTheme.prototype, "readme", void 0);
exports.default = MarkdownTheme;
